package id.durianpay.dpay_flutter;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Map;

import io.flutter.plugin.common.MethodChannel.Result;

import id.durianpay.android.Durianpay;
import id.durianpay.android.Interfaces.CheckoutResultListener;
import id.durianpay.android.model.DCheckoutOptions;
import id.durianpay.android.model.DPaymentFailed;
import id.durianpay.android.model.DPaymentSuccess;
import com.google.gson.Gson;

@RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
public class DurianpayCheckout implements CheckoutResultListener {
    private Activity activity;
    private Result result;

    //initialize the activity from DpayFlutterPlugin
    public DurianpayCheckout(Activity act) {
        activity = act;
    }

    //open Dpay Checkout using the checkout options
    public void openCheckout(final Map<String, Object> checkoutoptions, final Result result) {
        this.result = result;
        //use main thread to open the webview
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Log.d("into the class", "DurianpayCheckout");
                final JSONObject options = new JSONObject(checkoutoptions);

                //call android sdk to open the webview
                final DCheckoutOptions checkoutOptions = new DCheckoutOptions();
                CheckoutMapper.map(options, checkoutOptions); //validate all the options
                Durianpay.getInstance(activity).checkout(checkoutOptions, DurianpayCheckout.this);
            }
        });
    }

    //callback for transaction succeess from android native and send a callback to dart native
    @SuppressLint("LongLogTag")
    @Override
    public void onSuccess(final DPaymentSuccess dPaymentSuccess) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {                
                String success = new Gson().toJson(dPaymentSuccess);
                Log.d("from android-sdk to flutter android", success);                
                result.success(success);
            }
        });
    }

    //callback for failed transaction from android native and send a callback to dart native
    @SuppressLint("LongLogTag")
    @Override
    public void onFailure(final DPaymentFailed dPaymentFailed) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                String failed = new Gson().toJson(dPaymentFailed);
                Log.d("from android-sdk to flutter android", failed);
                result.success(failed);
            }
        });
    }


    //callback for closed transaction from android native and send a callback to dart native
    @SuppressLint("LongLogTag")
    @Override
    public void onClose(final String s) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Log.d("from android-sdk to flutter android", s);
                result.success(s);
            }
        });
    }
}
