import 'dart:async';
import 'dart:convert';

import 'package:dpay_flutter/CheckoutOptions.dart';
import 'package:eventify/eventify.dart';
import 'package:flutter/services.dart';
import 'package:flutter/cupertino.dart';
import 'package:logger/logger.dart';

import 'Constants.dart';

class Durianpay {
  //Bridge/channel between dart native and platform specific code
  static const MethodChannel _channel = const MethodChannel('dpay_flutter');
  var logger = Logger(); //logger initialization
  BuildContext? globalContext;
// EventEmitter instance used for communication
  EventEmitter? _eventEmitter;

  static Durianpay? _instance; //instance of durianpay class
  //initialization
  Durianpay._(BuildContext context) {
    _eventEmitter = new EventEmitter();
    globalContext = context;
  }

  //singleton instance of Durianpay class
  static getInstance(BuildContext context) {
    if (Durianpay._instance != null) {
      return Durianpay._instance;
    }

    Durianpay._instance = new Durianpay._(context);

    return Durianpay._instance;
  }

  //checkout function to open platform specific code
  checkout(DCheckoutOptions options) async {
    logger.d(options.toMap());
    logger.d("response before invoking method in flutter");
    String response = await (_channel.invokeMethod(
        "openCheckout",
        options
            .toMap())); //call openCheckout function in platform specific code with checkout options
    logger.d("response after invoking in flutter");
    logger.d("The response is " + response);
    handleResponse(response); //handle the callback result
  }

  /// Registers event listeners for payment events
  void on(String event, Function handler) {
    EventCallback callBack = (event, context) {
      handler(event.eventData);
    };
    _eventEmitter!.on(event, null, callBack);
  }

  /// Clears all event listeners
  void clear() {
    if (_eventEmitter != null) {
      _eventEmitter!.clear();
    }
  }

  //send callback to merchant/client application according to the transaction response
  void handleResponse(String response) {
    var body = json.decode(response);
    if (_eventEmitter != null) {
      switch (body["type"]) {
        case "success":
          _eventEmitter!.emit(PAYMENT_SUCCESS, null, response);
          break;
        case "failure":
          _eventEmitter!.emit(PAYMENT_FAIL, null, response);
          break;
        case "close":
          _eventEmitter!.emit(PAYMENT_CLOSE, null, response);
          break;
      }
    }
  }
}
